var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * @memberof utilities
 * @namespace propUtilities
 * @author Steven Jackson
 */
import _ from "lodash";
import PropTypes from "prop-types";
import elementType from "prop-types-extra/lib/elementType";
import isRequiredForA11y from "prop-types-extra/lib/isRequiredForA11y";
import cx from "classnames";
import { ROLE, getParentClass } from "./ROLE.js";
import { uID } from "./coreUtilities.js";
import UIGlobals from "./UIGlobals.js";
/**
 * [OBSERVABLE_EVENTS description]
 * @type {Array}
 */
var OBSERVABLE_EVENTS = ["scroll", "wheel", "mousemove", "keypress", "copy", "paste", "touchmove", "touchstart", "error", "load", "toggle"];
/**
 * [NATIVE_PROPS description]
 * @type {Array}
 */
var NATIVE_PROPS = ["id", "name", "type", "value", "checked", "defaultChecked", "required", "className", "role", "style", "href", "target", "src", "width", "height", "colSpan", "children", "onClick", "onSelect", "onChange", "onBlur", "onFocus", "onMouseDown", "onMouseUp", "onMouseEnter", "onMouseLeave", "onKeyDown", "onKeyUp", "onToggle", "onSubmit"];
/**
 * [INHERITED_PROPS description]
 * @type {Array}
 */
var INHERITED_PROPS = ["uiclass", "path", "disabled", "uidata", "invalid"];
/**
 * [DefaultPropTypes description]
 * @type {Object}
 */
var DefaultPropTypes = {
  uuid: PropTypes.string,
  id: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  name: PropTypes.string,
  type: PropTypes.string,
  value: PropTypes.string,
  label: PropTypes.string,
  loader: PropTypes.oneOf(["cube", "sheep"]),
  reversed: PropTypes.bool,
  invalid: PropTypes.bool,
  checked: PropTypes.bool,
  defaultChecked: PropTypes.bool,
  required: PropTypes.bool,
  inset: PropTypes.bool,
  raised: PropTypes.bool,
  reveal: PropTypes.bool,
  well: PropTypes.bool,
  panel: PropTypes.bool,
  rounded: PropTypes.bool,
  circular: PropTypes.bool,
  masked: PropTypes.bool,
  renderAs: elementType,
  className: PropTypes.string,
  role: PropTypes.string,
  group: PropTypes.string,
  tooltip: PropTypes.string,
  children: PropTypes.node,
  uiclass: PropTypes.string,
  uirole: PropTypes.string,
  uigroup: PropTypes.string,
  path: PropTypes.string,
  text: PropTypes.string,
  icon: PropTypes.string,
  to: PropTypes.string,
  disabled: PropTypes.bool,
  collapsed: PropTypes.bool,
  open: PropTypes.bool,
  closed: PropTypes.bool,
  hidden: PropTypes.bool,
  shown: PropTypes.bool,
  active: PropTypes.bool,
  overlay: PropTypes.bool,
  fixed: PropTypes.bool,
  vertical: PropTypes.bool,
  fit: PropTypes.oneOf(["width", "height", "parent", "flex", "screen"]),
  color: PropTypes.oneOf(["transparent", "white", "grey", "black", "red", "orange", "yellow", "green", "blue", "indigo", "violet", "!white", "!grey", "!black", "!red", "!orange", "!yellow", "!green", "!blue", "!indigo", "!violet"]),
  colorStyle: PropTypes.oneOf(["fill", "outline", "text"]),
  colorHover: PropTypes.bool,
  displaySize: PropTypes.oneOf(["smallest", "small", "medium", "large", "largest"]),
  align: PropTypes.oneOf(["left", "right", "center"]),
  textAlign: PropTypes.oneOf(["left", "right", "center"]),
  contentAlign: PropTypes.oneOf(["left", "right", "center"]),
  position: PropTypes.oneOf(["left", "before", "right", "after", "top", "above", "bottom", "below"]),
  shadow: PropTypes.string,
  observe: PropTypes.oneOf(OBSERVABLE_EVENTS),
  dispatch: PropTypes.oneOf(OBSERVABLE_EVENTS),
  uidata: PropTypes.object
};
/**
 * [DefaultPropValues description]
 * @type {Object}
 */
var DefaultPropValues = {
  uuid: null,
  renderAs: "div",
  className: null,
  role: null,
  name: null,
  type: null,
  label: null,
  loader: null,
  value: null,
  reversed: null,
  invalid: null,
  checked: null,
  defaultChecked: null,
  required: null,
  group: null,
  tooltip: null,
  children: null,
  uiclass: null,
  uirole: "",
  uigroup: ROLE.UI,
  path: "/",
  text: null,
  icon: null,
  inset: null,
  raised: null,
  reveal: null,
  well: null,
  panel: null,
  rounded: null,
  circular: null,
  masked: null,
  to: null,
  disabled: null,
  collapsed: null,
  open: null,
  closed: null,
  hidden: null,
  shown: null,
  active: null,
  overlay: null,
  fixed: null,
  vertical: null,
  fit: null,
  color: null,
  colorStyle: null,
  colorHover: null,
  displaySize: null,
  align: null,
  textAlign: null,
  contentAlign: null,
  position: null,
  shadow: null,
  observe: null,
  dispatch: null,
  uidata: {}
};
/**
 * [stringToPropType description]
 * @param  {String} propString [description]
 * @return {String}            [description]
 */
function stringToPropType(propString) {
  var required = String(propString).indexOf("!") !== -1;
  var prop = String(propString).replace("!", "");
  if (typeof PropTypes[prop] !== "undefined") {
    return required ? PropTypes[prop].isRequired : PropTypes[prop];
  }
  return PropTypes.string;
}
/**
 * [assignPropType description]
 * @param  {String} propVal [description]
 * @return {String}         [description]
 */
function assignPropType(propVal) {
  if (_.isArray(propVal)) {
    var assignMethod = "oneOf";
    var typeValues = propVal.map(function (val) {
      if (typeof PropTypes[val] !== "undefined") {
        assignMethod = "oneOfType";
        return stringToPropType(val);
      }
      return val;
    });
    if (typeof PropTypes[assignMethod] !== "undefined") {
      return PropTypes[assignMethod](typeValues);
    }
  }
  return stringToPropType(propVal);
}
/**
 * [setPropTypes description]
 * @param {Boolean} [isA11y=false] [description]
 * @param {String}  config         [description]
 * @param {String}  uidataConfig   [description]
 */
function setPropTypes() {
  var isA11y = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var config = arguments[1];
  var uidataConfig = arguments[2];

  var obj = _.clone(DefaultPropTypes);
  if (isA11y) {
    obj.id = isRequiredForA11y(PropTypes.oneOfType([PropTypes.string, PropTypes.number]));
  }
  if (typeof config !== "undefined" && config !== null) {
    Object.entries(config).forEach(function (_ref) {
      var attr = _ref[0],
          val = _ref[1];

      obj[attr] = assignPropType(val);
    });
  }
  if (typeof uidataConfig !== "undefined" && uidataConfig !== null) {
    var uidata = {};
    Object.entries(uidataConfig).forEach(function (_ref2) {
      var attr = _ref2[0],
          val = _ref2[1];

      uidata[attr] = assignPropType(val);
    });
    obj.uidata = PropTypes.shape(uidata);
  }
  return obj;
}
/**
 * [setCorePropTypes description]
 * @param {String} config       [description]
 * @param {String} uidataConfig [description]
 */
export function setCorePropTypes(config, uidataConfig) {
  return setPropTypes(false, config, uidataConfig);
}
/**
 * [setPropTypesA11y description]
 * @param {String} config       [description]
 * @param {String} uidataConfig [description]
 */
export function setPropTypesA11y(config, uidataConfig) {
  return setPropTypes(true, config, uidataConfig);
}
/**
 * [setPropDefaults description]
 * @param {String} autoId       [description]
 * @param {String} config       [description]
 * @param {String} uidataConfig [description]
 */
function setPropDefaults(autoId, config, uidataConfig) {
  var obj = _.clone(DefaultPropValues);
  obj.uuid = uID();
  if (typeof config !== "undefined" && config !== null) {
    Object.entries(config).forEach(function (_ref3) {
      var attr = _ref3[0],
          prop = _ref3[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        obj[attr] = _.clone(prop);
      } else {
        obj[attr] = prop;
      }
    });
  }
  if (typeof uidataConfig !== "undefined" && uidataConfig !== null) {
    var d = {};
    Object.entries(uidataConfig).forEach(function (_ref4) {
      var attr = _ref4[0],
          prop = _ref4[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        d[attr] = _.clone(prop);
      } else {
        d[attr] = prop;
      }
    });
    obj.uidata = d;
    obj.privatedata = d;
  }
  if (_.isNil(obj.uiclass) && !_.isNil(obj.uirole)) {
    obj.uiclass = obj.uirole;
  }
  if (autoId && _.isNil(obj.id)) {
    obj.id = obj.uirole + "_" + obj.uuid;
  }
  return obj;
}
/**
 * [setCorePropDefaults description]
 * @param {String} config       [description]
 * @param {String} uidataConfig [description]
 */
export function setCorePropDefaults(config, uidataConfig) {
  return setPropDefaults(false, config, uidataConfig);
}
/**
 * [setPropDefaultsAutoId description]
 * @param {String} config       [description]
 * @param {String} uidataConfig [description]
 */
export function setPropDefaultsAutoId(config, uidataConfig) {
  return setPropDefaults(true, config, uidataConfig);
}
/**
 * [getUIClassString description]
 * @param  {String} props [description]
 * @param  {String} state [description]
 * @return {String}       [description]
 */
export function getUIClassString(props, state) {
  var _coreClasses, _styleClasses, _iconClass;

  var uiclass = props.uiclass,
      className = props.className,
      overlay = props.overlay,
      fixed = props.fixed,
      inset = props.inset,
      raised = props.raised,
      reveal = props.reveal,
      well = props.well,
      panel = props.panel,
      rounded = props.rounded,
      circular = props.circular,
      masked = props.masked,
      fit = props.fit,
      color = props.color,
      colorStyle = props.colorStyle,
      colorHover = props.colorHover,
      textAlign = props.textAlign,
      contentAlign = props.contentAlign,
      anchor = props.anchor,
      shadow = props.shadow,
      icon = props.icon,
      reversed = props.reversed,
      vertical = props.vertical,
      loader = props.loader;


  var disabled = state && !_.isNil(state.disabled) ? state.disabled : props.disabled;
  var active = state && !_.isNil(state.active) ? state.active : props.active;
  var collapsed = state && !_.isNil(state.collapsed) ? state.collapsed : props.collapsed;
  var open = state && !_.isNil(state.open) ? state.open : props.open;
  var closed = state && !_.isNil(state.closed) ? state.closed : props.closed;
  var hidden = state && !_.isNil(state.hidden) ? state.hidden : props.hidden;
  var shown = state && !_.isNil(state.shown) ? state.shown : props.shown;
  var invalid = state && !_.isNil(state.invalid) ? state.invalid : props.invalid;

  var sUIclass = "ui-" + uiclass;
  var hasUIclass = !className || className.indexOf(sUIclass) === -1;
  var coreClasses = (_coreClasses = {}, _coreClasses[sUIclass] = hasUIclass, _coreClasses["ui-anchor-" + anchor] = anchor, _coreClasses["ui-content-align-" + contentAlign] = contentAlign, _coreClasses["ui-text-align-" + textAlign] = textAlign, _coreClasses["ui-fit-" + fit] = fit, _coreClasses["ui-overlay"] = overlay, _coreClasses["ui-fixed"] = fixed, _coreClasses["ui-inset"] = inset, _coreClasses["ui-raised"] = raised, _coreClasses["ui-reveal"] = reveal, _coreClasses["ui-rounded"] = rounded, _coreClasses["ui-circular"] = circular, _coreClasses["ui-well"] = well, _coreClasses["ui-panel"] = panel, _coreClasses["ui-masked"] = masked, _coreClasses["ui-shadow"] = shadow, _coreClasses["ui--active"] = active, _coreClasses["ui--locked"] = !!loader, _coreClasses["ui--vertical"] = vertical, _coreClasses["ui--reversed"] = reversed, _coreClasses["ui--collapsed"] = collapsed, _coreClasses["ui--open"] = open, _coreClasses["ui--closed"] = closed, _coreClasses["ui--hidden"] = hidden, _coreClasses["ui--shown"] = shown, _coreClasses["ui--disabled"] = disabled, _coreClasses["ui--invalid"] = invalid, _coreClasses);

  var theme = UIGlobals.readSetting("theme");
  var styleClasses = (_styleClasses = {}, _styleClasses[sUIclass + "-theme-" + theme] = uiclass && theme, _styleClasses);

  if (!_.isNil(color)) {
    var cleanColor = String(color).replace(/[^\w]/g, "");
    var colorClass = "ui-" + cleanColor;
    if (!_.isNil(colorStyle)) {
      colorClass += "-" + colorStyle;
    }
    styleClasses[colorClass] = true;
  }
  var iconPre = icon && icon.indexOf("ui-icon-") !== -1 ? icon : "ui-icon ui-icon-" + icon;
  var iconClass = (_iconClass = {}, _iconClass[iconPre] = icon, _iconClass);
  // if (!_.isNil(anchor)) {
  //   classes[`ui-anchor-${anchor}`] = true;
  // }
  // if (!_.isNil(contentAlign)) {
  //   classes[`ui-content-align-${contentAlign}`] = true;
  // }
  // if (!_.isNil(textAlign)) {
  //   classes[`ui-text-align-${textAlign}`] = true;
  // }
  var uiClassCore = cx(className, coreClasses);
  var uiClassIcon = cx("", iconClass);
  var uiClassStyle = cx(uiClassIcon, styleClasses);
  var uiClassFull = cx(uiClassCore, uiClassStyle);
  return [uiClassFull, uiClassCore, uiClassStyle, uiClassIcon];
}
/**
 * [getValidProps description]
 * @param  {String} source [description]
 * @param  {String} state  [description]
 * @return {String}        [description]
 */
export function getValidProps(source, state) {
  var obj = { props: {}, inherited: {} };
  if (source.style) {
    var uiStyle = {};
    Object.entries(source.style).forEach(function (_ref5) {
      var attr = _ref5[0],
          prop = _ref5[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        uiStyle[attr] = _.clone(prop);
      } else {
        uiStyle[attr] = prop;
      }
    });
    obj.props.style = uiStyle;
  }
  Object.entries(source).forEach(function (_ref6) {
    var attr = _ref6[0],
        value = _ref6[1];

    if (attr !== "style") {
      if (NATIVE_PROPS.indexOf(attr) !== -1 && value !== null) {
        obj[attr] = value;
        obj.props[attr] = value;
      } else if (attr !== "uidata" && attr !== "privatedata") {
        if ((typeof value === "undefined" ? "undefined" : _typeof(value)) === "object") {
          obj[attr] = _.clone(value);
        } else {
          obj[attr] = value;
        }
      }
      if (INHERITED_PROPS.indexOf(attr) !== -1 && !_.isNil(value)) {
        obj.inherited[attr] = value;
      }
    }
  });
  if (typeof source.privatedata !== "undefined") {
    obj.uidata = _.clone(source.privatedata, source.uidata);
  }
  var uiClassNames = getUIClassString(source, state);
  obj.className = uiClassNames[0];
  obj.coreClassName = uiClassNames[1];
  obj.styleClassName = uiClassNames[2];
  obj.iconClassName = uiClassNames[3];
  obj.props.className = uiClassNames[0];
  if (_.isNil(obj.props.key)) {
    obj.props.key = "component-" + obj.props.uuid;
  }
  if (_.isNil(obj.parentclass)) {
    obj.parentclass = getParentClass(obj);
  }
  obj.inherited.parentclass = obj.parentclass;
  if (_.isNil(obj.inherited.uiclass)) {
    obj.inherited.uiclass = "";
  }
  return obj;
}